/* sxplorer:
 * Copyright 1996 Y.YONEYAMA(fj9y-ynym@asahi-net.or.jp)
 */
#include "sxplorer.h"
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>

/* sxplorer.c */
extern Widget root;
extern char TrashFolder[];
extern int display_width, display_height;

#define EXIT_ALERT      "Really Quit?"
#define DESTROY_ALERT   "Destroy Trash?"
#define PERMIT_ALERT    "Permission denied."
#define OVERWRITE_ALERT "Overwrite %s?"
#define REMOVE_ALERT    "Remove %s?"
#define RMALIAS_ALERT   "Remove alias %s?"


typedef struct {
  char *from;
  char *to;
  Icon *icon;
  Icon *folder;
  void (*func)();
} CopyFiles;

void Centering(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Window r;
  int x, y;
  unsigned int width, height, boder, depth;
  XGetGeometry(XtDisplay(w), XtWindow(w), &r, &x, &y,
       &width, &height, &boder, &depth);
  XMoveWindow(XtDisplay(w), XtWindow(w), (display_width - width)/2,
      (display_height - height)/2);
}

typedef struct {
  Widget *widget;
  void (*callback)();
  XtPointer data;
} OkData;

void AlertDialog(msg, okcallback, data)
char *msg;
void (*okcallback)();
XtPointer data;
{
  Widget *msgdialog;
  Widget form;
  Widget label, ok, cancel;
  void OkCallback();
  void CancelCallback();
  static XtActionsRec actions[] = {{"Centering", Centering}};
  String translations = "<Create>: Centering()";

  msgdialog = XtNew(Widget);
  XtAddActions(actions, XtNumber(actions));
  *msgdialog = XtVaCreatePopupShell(PROGRAM_NAME, transientShellWidgetClass,
      root, XtNx, display_width, XtNy, display_height,
      XtNtranslations, XtParseTranslationTable(translations), NULL);
  form = XtVaCreateManagedWidget(NULL, formWidgetClass,
      *msgdialog , XtNborderWidth, 0, NULL);
  label = XtVaCreateManagedWidget("dialog", labelWidgetClass,
      form, XtNlabel, msg, XtNborderWidth, 0, NULL);
  ok = XtVaCreateManagedWidget("dialog", commandWidgetClass,
      form, XtNfromVert, label, XtNlabel, "OK", NULL);
  if(okcallback == NULL)
  {
    XtAddCallback(ok, XtNcallback, CancelCallback, msgdialog);
  }
  else
  {
    OkData *okdata;
    okdata = XtNew(OkData);
    okdata->widget = msgdialog;
    okdata->callback = okcallback;
    okdata->data = data;
    XtAddCallback(ok, XtNcallback, OkCallback, okdata);
    cancel = XtVaCreateManagedWidget("dialog", commandWidgetClass,
        form, XtNfromVert, label,
        XtNfromHoriz, ok, XtNlabel, "Cancel", NULL);
    XtAddCallback(cancel, XtNcallback, CancelCallback, msgdialog);
  }

  XtPopup(*msgdialog, XtGrabExclusive);
  XBell(XtDisplay(root), 0);
}

void OkCallback(w, client_data, call_data)
Widget w;
OkData *client_data;
caddr_t call_data;
{
  (*client_data->callback)(w, client_data->data, NULL);
  XtPopdown(*client_data->widget);
  XtFree((char *)client_data->widget);
  XtFree((char *)client_data);
}
void CancelCallback(w, client_data, call_data)
Widget w;
Widget *client_data;
caddr_t call_data;
{
  XtPopdown(*client_data);
  XtFree((char *)client_data);
}

void ExitCallback(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
  quit();
}

void DestroyCallback(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
  RemoveFile(TrashFolder);
  DestroyIconAll(TrashFolder);
  MakeTrashFolder(TrashFolder);
  RedrawTrashIcons();
}

void OverwriteCallback(w, data, call_data)
Widget w;
CopyFiles *data;
caddr_t call_data;
{
#ifdef DEBUG
printf("OverwriteCallback: %s -> %s\n", data->from, data->to);
#endif
  data->func(data->from, data->to, data->icon, data->folder);
  XtFree(data->from);
  XtFree(data->to);
}

void RemoveCallback(w, data, call_data)
Widget w;
char *data;
caddr_t call_data;
{
#ifdef DEBUG
printf("RemoveCallback: %s\n", data);
#endif
  RemoveFile(data);
  DestroyIconAll(data);
  XtFree(data);
}

void RemoveAliasCallback(w, data, call_data)
Widget w;
Icon *data;
caddr_t call_data;
{
#ifdef DEBUG
printf("RemoveAliasCallback: %s\n", data->name);
#endif
  DestroyIcon(data->widget, NULL, NULL, 0);
}

void ExitDialog()
{
  AlertDialog(EXIT_ALERT, ExitCallback, NULL);
}

void DestroyDialog()
{
  AlertDialog(DESTROY_ALERT, DestroyCallback, NULL);
}

void PermitDialog()
{
  AlertDialog(PERMIT_ALERT, NULL, NULL);
}

void OverwriteDialog(from, to, icon, folder, func)
char *from;
char *to;
Icon *icon, *folder;
void (*func)();
{
  char msg[256];
  static CopyFiles data;
  sprintf(msg, OVERWRITE_ALERT, to);
  data.from = XtNewString(from);
  data.to = XtNewString(to);
  data.icon = icon;
  data.folder = folder;
  data.func = func;
  AlertDialog(msg, OverwriteCallback, &data);
}

void RemoveDialog(fname)
char *fname;
{
  char msg[256];
  sprintf(msg, REMOVE_ALERT, fname);
  AlertDialog(msg, RemoveCallback, XtNewString(fname));
}

void RemoveAliasDialog(icon)
Icon *icon;
{
  char msg[256];
  sprintf(msg, RMALIAS_ALERT, icon->name);
  AlertDialog(msg, RemoveAliasCallback, icon);
}
