/* sxplorer:
 * Copyright 1996 Y.YONEYAMA(fj9y-ynym@asahi-net.or.jp)
 */
#include "sxplorer.h"
#include <dirent.h>

/* #define SAFE_MODE */

/* sxplorer.c */
extern char HomeFolder[];
extern char TrashFolder[];


#define MAX_FILETYPE 256
struct _FileType {
  char *str;
  int  picture;
} FileType[MAX_FILETYPE];
int _nfiletype = 0;

char *GetFileName(path)
char *path;
{
  int i;
  for(i=strlen(path)-1;i>0;i--)
    if(path[i] == '/') break;
  if(path[i] == '/') i++;
  return(&path[i]);
}

char *GetExt(name)
char *name;
{
  int i;
  for (i=strlen(name)-1;i>0;i--)
  {
    if(name[i] == '.') return(&name[i]);
  }
  return(name);
}

char *GetDirectory(dst, path)
char *dst;
char *path;
{
  int i;
  strcpy(dst, path);
  for(i=strlen(dst)-1;i>0;i--)
    if(dst[i] == '/') break;
  if(i == 0) dst[i++] = '/';
  dst[i] = '\0';
  return(dst);
}

int CompareFileName(pat, path)
char *pat;
char *path;
{
  char *pn;
  pn = GetFileName(path);

  if(pat[0] == '*')
  {
    pat++;
    if(strlen(pn) - 1 < strlen(pat))
      return(-1);
    if(strcmp(&pn[strlen(pn)-strlen(pat)], pat) == 0)
      return(0);
  }
  else if(strcmp(pat, pn) == 0)
    return(0);
  return(-1);
}

Boolean IsFolder(mode)
mode_t mode;
{
  if(mode == NOTEXISTS) return(False);
  if ((mode & S_IFDIR) == S_IFDIR) return(True);
  return(False);
}

Boolean IsExecutable(mode)
mode_t mode;
{
  if(mode == NOTEXISTS) return(False);
  if ((mode & S_IXUSR) == S_IXUSR) return(True);
  return(False);
}

Boolean IsSymbolicLink(mode)
mode_t mode;
{
  if(mode == NOTEXISTS) return(False);
  if ((mode & S_IFLNK) == S_IFLNK) return(True);
  return(False);
}

time_t GetATime(path)
char *path;
{
  struct stat stbuf;
  if(lstat(path, &stbuf) == 0)
    return(stbuf.st_atime);
  else
    return(-1);
}
time_t GetMTime(path)
char *path;
{
  struct stat stbuf;
  if(lstat(path, &stbuf) == 0)
    return(stbuf.st_mtime);
  else
    return(-1);
}


mode_t GetMode(path)
char *path;
{
  struct stat stbuf;
  if(lstat(path, &stbuf) == 0)
    return(stbuf.st_mode);
  else
    return(NOTEXISTS);
}

#define MAX_LINKS 10   /* not use "LINK_MAX" */

void GetRealPath(dst, path)
char *dst;
char *path;
{
  char lfile[1024];
  char lpath[1024];
  char nextpath[1024];
  int i, n;
  mode_t mode;

  strcpy(lpath, path);
  for(i=0;i<MAX_LINKS;i++)
  {
    mode = GetMode(lpath);
    if(mode == NOTEXISTS)
      break;
    if(!IsSymbolicLink(mode))
    {
      strcpy(dst, lpath);
      return;
    }
    n = readlink(lpath, lfile, 1024);
    if(n > 0)
      lfile[n] = '\0';
    else
      break;
    if(lfile[0] != '/')
    {
      GetDirectory(nextpath, lpath);
      strcat(nextpath, "/");
      strcat(nextpath, lfile);
      strcpy(lpath, nextpath);
    }
    else
      strcpy(lpath, lfile);
  }
  strcpy(dst, "");
  return;
}

mode_t GetLMode(path)
char *path;
{
  char lpath[1024];

  GetRealPath(lpath, path);
  if(strlen(lpath) == 0) return(NOTEXISTS);
  return(GetMode(lpath));
}

void AddFileType(str, iconname)
char *str;
char *iconname;
{
  if(_nfiletype >= MAX_FILETYPE) return;
  FileType[_nfiletype].str = XtNewString(str);
  FileType[_nfiletype].picture = GetIconData(iconname);
  _nfiletype ++;
}

int GetFileType(path)
char *path;
{
  mode_t mode;
  int i;
  char *pn;
  mode = GetMode(path);
  if(mode == NOTEXISTS) return(GetIconData(ICON_BROKEN));
  pn = GetFileName(path);
  for(i=_nfiletype - 1; i >= 0; i--)
  {
    if(CompareFileName(FileType[i].str, pn) == 0)
        return(FileType[i].picture);
  }
  if(IsExecutable(mode))
    return(GetIconData(ICON_EXEC));
  return(GetIconData(ICON_TEXT));
}

int copy(from, to)
char *from;
char *to;
{
  FILE *ifp, *ofp;
  char buf[1024];
  int len;
  mode_t mode;

  mode = GetMode(from);

  ofp = fopen(to, "wb");
  if(ofp == NULL)
    return(-1);
  ifp = fopen(from, "rb");
  if(ifp == NULL)
    return(-1);
  do
  {
    len = fread(buf, 1, 1024, ifp);
    if(len > 0)
      if(fwrite(buf, 1, len, ofp) < len)
      {
        fclose(ofp);
        fclose(ifp);
        remove(to);
        return(-1);
      }
  } while(len == 1024);
  fclose(ofp);
  fclose(ifp);
  chmod(to, mode);
  return(0);
}


Boolean IsValidFile(fname, flag)
char *fname;
Boolean flag;
{
  if (strcmp(fname, "..") == 0 ||
      strcmp(fname, ".") == 0) return(False);
  if(flag)
    if(fname[0] == '.') return(False);
  if(strcmp(fname, TRASH_FOLDER) == 0) return(False);
  return(True);
}

Boolean CopyFile(from, to)
char *from;
char *to;
{
  mode_t mode;
  Boolean RemoveFile();

#ifdef DEBUG
printf("CopyFile(%s,%s)\n", from, to);
#endif

  if(strcmp(from, to) == 0)
    return(False);

  if(strncmp(from, to, strlen(from)) == 0) /* loop copy... */
  {
    if(strlen(to) > strlen(from) && to[strlen(from)] == '/')
      return(False);
  }

#ifdef SAFE_MODE
  if(strncmp(to, HomeFolder, strlen(HomeFolder)) != 0)
  {
    return(False);
  }
#endif

  mode = GetMode(from);
  if(mode == NOTEXISTS)
    return(False);
  if(IsSymbolicLink(mode))
  {
    char lpath[1024];
    int n;
    n = readlink(from, lpath, 1024);
    if(n > 0)
      lpath[n] = '\0';
    else
      return(False);
    if(access(to, F_OK) == 0)
      if(RemoveFile(to) == False)
        return(False);
    if(symlink(lpath, to) != 0)
      return(False);
    return(True);
  }
  if(access(to, F_OK) == 0)
  {
    if(RemoveFile(to) == False)
      return(False);
  }
  if(IsFolder(mode))
  {
    DIR *dirp;
    struct dirent *dp;
    char newfrom[1024], newto[1024];
    dirp = opendir(from);
    if(dirp == NULL)
      return(False);
    if(mkdir(to, 0700) != 0) return(False);
    while(( dp = readdir(dirp)) != NULL)
    {
      if(IsValidFile(dp->d_name, 0) == False) continue;
      sprintf(newfrom, "%s/%s", from, dp->d_name);
      sprintf(newto, "%s/%s", to, dp->d_name);
      if(CopyFile(newfrom, newto) == False)
      {
        RemoveFile(to);
        closedir(dirp);
        return(False);
      }
    }
    closedir(dirp);
    chmod(to, mode);
  }
  else
  {
    if(copy(from, to) != 0)
      return(False);
#ifdef DEBUG
printf("Copy  %s -> %s\n", from, to);
#endif
  }
  return(True);
}

Boolean RemoveFile(from)
char *from;
{
  mode_t mode;

#ifdef DEBUG
printf("RemoveFile(%s)\n", from);
#endif

#ifdef SAFE_MODE
  if((strncmp(from, HomeFolder, strlen(HomeFolder)) != 0) &&
     (strncmp(from, "/tmp/", 5) != 0))
  {
    return(False);
  }
#endif

  mode = GetMode(from);
  if(mode == NOTEXISTS)
    return(False);
  if(IsSymbolicLink(mode))
  {
    if(remove(from) != 0)
      return(False);
    return(True);
  }
  if(IsFolder(mode))
  {
    DIR *dirp;
    struct dirent *dp;
    char fname[1024];
    if(chmod(from, 0755) != 0)
      return(False);
    dirp = opendir(from);
    if(dirp == NULL)
      return(False);
    while(( dp = readdir(dirp)) != NULL)
    {
      if(IsValidFile(dp->d_name, 0) == False) continue;
      sprintf(fname,"%s/%s",from,dp->d_name);
      if(RemoveFile(fname) == False)
      {
        closedir(dirp);
        return(False);
      }
    }
    closedir(dirp);
    if(rmdir(from) != 0)
      return(False);
    DestroyFolderAll(from);
#ifdef DEBUG
printf("RemoveDirectory %s\n", from);
#endif
  }
  else
  {
    if(remove(from) != 0)
      return(False);
#ifdef DEBUG
printf("Remove %s\n", from);
#endif
  }
  return(True);
}

Boolean MoveFile(from, to)
char *from;
char *to;
{
  mode_t mode;
#ifdef DEBUG
printf("MoveFile(%s, %s)\n", from, to);
#endif

  if(strcmp(from, to) == 0)
    return(False);

  if(strncmp(from, to, strlen(from)) == 0) /* loop copy... */
  {
    if(strlen(to) > strlen(from) && to[strlen(from)] == '/')
      return(False);
  }

#ifdef SAFE_MODE
  if(strncmp(from, HomeFolder, strlen(HomeFolder)) != 0 ||
     strncmp(to, HomeFolder, strlen(HomeFolder)) != 0)
  {
    return(False);
  }
#endif

  if(access(to, F_OK) == 0)
  {
    if(RemoveFile(to) == False)
      return(False);
  }
  mode = GetMode(from);
  if(rename(from, to) == 0)
  {
#ifdef DEBUG
printf("Rename %s -> %s\n", from, to);
#endif
  }
  else
  {
    if(CopyFile(from, to) == False)
      return(False);
    if(RemoveFile(from) == False)
    {
      RemoveFile(to);
      return(False);
    }
  }
  return(True);
}
Boolean MakeFolder(path)
char *path;
{
  if(mkdir(path, DEFAULT_FOLDERMODE) != 0)
    return(False);
  return(True);
}
Boolean MakeTrashFolder(path)
char *path;
{
  if(mkdir(path,  DEFAULT_TRASHMODE) != 0)
    return(False);
  return(True);
}

FileItem **SortItems(items, count, sort)
FileItem **items;
int count;
int sort;
{
  FileItem *tmp;
  int i, j;
  if(sort == 1)
  {
    for(i=0;i<count-1;i++)
    {
      for(j=i+1;j<count;j++)
      {
        if(strcmp(items[i]->name, items[j]->name) > 0)
        {
          tmp = items[j];
          items[j] = items[i];
          items[i] = tmp;
        }
      }
    }
  }
  return(items);
}

int CountFolderItems(path)
char *path;
{
  DIR *dirp;
  struct dirent *dp;
  int count = 0;
  dirp = opendir(path);
  if(dirp == NULL)
  {
    return(0);
  }
  while(( dp = readdir(dirp)) != NULL)
  {
    if (strcmp(dp->d_name, "..") == 0 ||
        strcmp(dp->d_name, ".") == 0) continue;
    count++;
  }
  closedir(dirp);
  return(count);
}

FileItem **GetFolderFolders(path, sort, flag)
char *path;
int sort;
Boolean flag;
{
  DIR *dirp;
  struct dirent *dp;
  mode_t mode;
  int count = 0, i;
  FileItem **folders;
  char fullpath[1024];

  dirp = opendir(path);
  if(dirp == NULL)
  {
    folders = (FileItem **)XtMalloc(sizeof(FileItem *));
    folders[0] = NULL;
    return(folders);
  }
  while(( dp = readdir(dirp)) != NULL)
  {
    if(IsValidFile(dp->d_name, flag) == False) continue;
    sprintf(fullpath,"%s/%s",path,dp->d_name);
    mode = GetMode(fullpath);
    if (mode == NOTEXISTS || !IsFolder(mode)) continue;
    count++;
  }
  rewinddir(dirp);
  folders = (FileItem **)XtMalloc(sizeof(FileItem *)*(count + 1));
  i = 0;
  while(( dp = readdir(dirp)) != NULL)
  {
    if(IsValidFile(dp->d_name, flag) == False) continue;
    sprintf(fullpath,"%s/%s",path,dp->d_name);
    mode = GetMode(fullpath);
    if (mode == NOTEXISTS || !IsFolder(mode)) continue;
    folders[i] = XtNew(FileItem);
    folders[i]->name = XtNewString(dp->d_name);
    i++;
  }
  folders[i] = NULL;
  closedir(dirp);
  if(sort != 0)
    SortItems(folders, count, sort);
  return(folders);
}

FileItem **GetFolderFiles(path, sort, flag)
char *path;
int sort;
Boolean flag;
{
  DIR *dirp;
  struct dirent *dp;
  mode_t mode;
  int count = 0, i;
  FileItem **files;
  char fullpath[1024];

  dirp = opendir(path);
  if(dirp == NULL)
  {
    files = (FileItem **)XtMalloc(sizeof(FileItem *));
    files[0] = NULL;
    return(files);
  }
  while(( dp = readdir(dirp)) != NULL)
  {
    if(IsValidFile(dp->d_name, flag) == False) continue;
    sprintf(fullpath,"%s/%s",path,dp->d_name);
    mode = GetMode(fullpath);
    if (mode == NOTEXISTS || IsFolder(mode)) continue;
    count++;
  }
  rewinddir(dirp);
  files = (FileItem **)XtMalloc(sizeof(FileItem *)*(count + 1));
  i = 0;
  while(( dp = readdir(dirp)) != NULL)
  {
    if(IsValidFile(dp->d_name, flag) == False) continue;
    sprintf(fullpath,"%s/%s",path,dp->d_name);
    mode = GetMode(fullpath);
    if (mode == NOTEXISTS || IsFolder(mode)) continue;
    files[i] = XtNew(FileItem);
    files[i]->name = XtNewString(dp->d_name);
    i++;
  }
  files[i] = NULL;
  closedir(dirp);
  if(sort > 0)
    SortItems(files, count, sort);
  return(files);
}

p_mode GetPMode(path)
char *path;
{
  mode_t mode;
  p_mode p = 0;
  char fn[1024];

  if(access(path, F_OK) != 0)
    return(0);

  mode = GetMode(path);
  if(IsSymbolicLink(mode))
  {
    p |= P_LINK;
    GetRealPath(fn, path);
  }
  else
  {
    strcpy(fn, path);
  }
  mode = GetMode(fn);
  if(mode == NOTEXISTS)
    return(p);
  if(IsFolder(mode))
    p |= P_FOLDER;
  
  if(access(fn, R_OK) == 0)
    p |= P_READ;
  if(access(fn, W_OK) == 0)
    p |= P_WRITE;
  if(access(fn, X_OK) == 0)
    p |= P_EXEC;

  return(p);
}

void ScanFolder(folder, path, flag)
Folder *folder;
char *path;
Boolean flag;
{
  mode_t mode;
  char fullpath[1024];
  int i;
  FileItem **folders;
  FileItem **files;

  /* scan directories */
  folders = GetFolderFolders(path, 1, flag);
  for(i=0;folders[i]!=NULL;i++)
  {
    sprintf(fullpath,"%s/%s", path, folders[i]->name);
    CreateIcon(folder->base, 0, 0, folders[i]->name, fullpath, "",
        GetIconData(ICON_FOLDER), GetPMode(fullpath));
  }

  /* scan files */
  files = GetFolderFiles(path, 1, flag);
  for(i=0;files[i]!=NULL;i++)
  {
    if(strcmp(path, "/") == 0)
      sprintf(fullpath,"/%s",files[i]->name);
    else
      sprintf(fullpath,"%s/%s",path,files[i]->name);
    mode = GetMode(fullpath);
    if (IsSymbolicLink(mode))
      CreateIcon(folder->base,0,0, files[i]->name, fullpath, "",
          GetIconData(ICON_LINK), P_LINK);
    else if (IsExecutable(mode))
        CreateIcon(folder->base,0,0, files[i]->name, fullpath, "",
           GetFileType(fullpath), GetPMode(fullpath));
    else
        CreateIcon(folder->base,0,0, files[i]->name, fullpath, "",
           GetFileType(fullpath), GetPMode(fullpath));
  }
  folder->folders = folders;
  folder->files = files;
}

