/* sxplorer:
 * Copyright 1996 Y.YONEYAMA(fj9y-ynym@asahi-net.or.jp)
 */
#include "sxplorer.h"
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Simple.h>
#include <X11/cursorfont.h>

/* sxplorer.c */
extern Widget root;
extern char TrashFolder[];
extern struct global_options GlobalOptions;
extern Atom wm_delete_window;
extern unsigned long fgcolor, bgcolor;

/* icon.c */
extern Icon *_icons[];
extern int _nicon;
extern Cursor stdcursor, waitcursor;
extern Boolean Dragging;

/* data.c */
extern struct _IconData IconData[];

#define MAX_FOLDERS 100
Folder *_folders[MAX_FOLDERS];
int _nfolder = 0;

time_t GetATime();

void CheckFolder();
void DragFolder();
void CloseFolder();
void ReconfigureFolder();
void DropToFolder();
void MappedFolder();
void IconifyFolder();

void ScanFolder();
void DestroyFolderIcons();

void InitFolder()
{
  static XtActionsRec actions[] =
  {  {"CloseFolder", CloseFolder},
     {"CheckFolder", CheckFolder},
     {"DragFolder", DragFolder},
     {"ReconfigureFolder", ReconfigureFolder},
     {"DropToFolder", DropToFolder},
     {"MappedFolder", MappedFolder},
     {"IconifyFolder", IconifyFolder},
  };
  XtAddActions(actions, XtNumber(actions));
}

Folder *GetFolder(w)
Widget w;
{
  int i;
  for (i=0;i<_nfolder;i++)
  {
    if (!_folders[i]->deleted &&
        (_folders[i]->widget == w ||
         _folders[i]->view   == w ||
         _folders[i]->base   == w))
      return(_folders[i]);
  }
  return(NULL);
}

Folder *IsOpenedFolder(path)
char *path;
{
  int i;
  for (i=0;i<_nfolder;i++)
  {
    if(!_folders[i]->deleted && strcmp(_folders[i]->path, path) == 0)
      return(_folders[i]);
  }
  return(NULL);
}

Folder *NewFolder(path)
char *path;
{
  int i;
  for (i=0;i<_nfolder;i++)
  {
    if(_folders[i]->deleted && strcmp(path, _folders[i]->path) == 0)
    { /* reopen */
      _folders[i]->deleted = False;
      return(_folders[i]);
    }
  }
  if(_nfolder < MAX_FOLDERS - 1)
  {
    int i;
    i = _nfolder;
    _nfolder++;
    _folders[i] = XtNew(Folder);
    _folders[i]->deleted = True;
    return(_folders[i]);
  }
  for (i=0;i<_nfolder;i++)
  {
    if(_folders[i]->deleted)
    {
      XtFree(_folders[i]->path);
      return(_folders[i]);
    }
  }
  return(NULL);
}

void OpenFolder(icon)
Icon *icon;
{
  Arg args[20];
  char path[1024];
  int n;
  static String translations = " \
     <FocusIn>:              CheckFolder() \n\
     <Configure>:            ReconfigureFolder() \n\
     <Unmap>:                IconifyFolder() \n\
     <Map>:                  MappedFolder() \n\
     <Message>WM_PROTOCOLS:  CloseFolder() \n\
     <Btn1Down>:             DragFolder() \n\
     <Btn1Up>:               DropToFolder()";
  Folder *f;
  int width, height;
  void GetRealPath();

  GetRealPath(path, icon->path);
  if(strlen(path) == 0)
    strcpy(path, icon->path);

  if((f = IsOpenedFolder(path)) != NULL)
  {
    if(f->minimized)
      XMapWindow(XtDisplay(f->widget), XtWindow(f->widget));
    XRaiseWindow(XtDisplay(f->widget), XtWindow(f->widget));
    if(f->minimized == False)
      XSetInputFocus(XtDisplay(root),
          XtWindow(f->widget), RevertToNone, CurrentTime);
    return;
  }

  f = NewFolder(path);
  f->minimized = True;
  n = 0;
  XtSetArg(args[n], XtNminWidth, 128); n++;
  XtSetArg(args[n], XtNminHeight, 64); n++;
  XtSetArg(args[n], XtNmaxWidth, DisplayWidth(XtDisplay(root), 0)); n++;
  XtSetArg(args[n], XtNmaxHeight, DisplayHeight(XtDisplay(root), 0)); n++;
  XtSetArg(args[n], XtNiconPixmap, IconData[icon->picture].picture); n++;
  XtSetArg(args[n], XtNiconMask, IconData[icon->picture].mask); n++;
  if (f->deleted)
  {
    int fcount;
    f->deleted = False;
    f->path = XtNewString(path);
    f->type = icon->type;
    fcount = CountFolderItems(f->path);
    if(fcount <= 6)
    {
      f->width = 264 + 16;
      f->height = 144;
    }
    else if(fcount <= 12)
    {
      f->width = 352 + 16;
      f->height = 216;
    }
    else if(fcount <= 20)
    {
      f->width = 440 + 16;
      f->height = 288;
    }
    else if(fcount <= 30)
    {
      f->width = 528 + 16;
      f->height = 360;
    }
    else
    {
      f->width = 562;
      f->height = 360;
    }
    XtSetArg(args[n], XtNwidth, f->width); n++;
    XtSetArg(args[n], XtNheight, f->height); n++;
  }
  else
  {
    XtSetArg(args[n], XtNwidth, f->width); n++;
    XtSetArg(args[n], XtNheight, f->height); n++;
  }
  if(ISPSYSTEM(icon->type) && strcmp(icon->path, NAME_SYSTEM) == 0)
  {
    char title[256];
    sprintf(title, "%s ver. %s", PROGRAM_NAME, VERSION);
    XtSetArg(args[n], XtNtitle, title); n++;
  }
  else
  {
    XtSetArg(args[n], XtNtitle, path); n++;
  }
  XtSetArg(args[n], XtNiconName, icon->name); n++;
  XtSetArg(args[n], XtNtranslations, 
      XtParseTranslationTable(translations)); n++;

  f->widget = XtCreateApplicationShell("folder",
      topLevelShellWidgetClass, args, n);
  n = 0;
  XtSetArg(args[n], XtNallowVert, True); n++;
  f->view = XtCreateManagedWidget("view", viewportWidgetClass,
      f->widget, args, n);
  n = 0;
  XtSetArg(args[n], XtNborderWidth, 0); n++;
  XtSetArg(args[n], XtNforeground, fgcolor); n++;
  XtSetArg(args[n], XtNbackground, bgcolor); n++;
  f->base = XtCreateManagedWidget("folder", boxWidgetClass,
      f->view, args, n);

  f->folders = NULL;
  f->files = NULL;

  if(IsTrashFolder(icon))
  {
    CreateIcon(f->base, 0, 0, "Destroy", NAME_DESTROY, "",
        GetIconData(ICON_DESTROY), P_SYSTEM | P_EXEC);
    if(GetMode(TrashFolder) == NOTEXISTS)
      MakeTrashFolder(TrashFolder);
    ScanFolder(f, TrashFolder, 0);
    f->atime = GetATime(TrashFolder);
  }
  else if(ISPSYSTEM(icon->type) && strcmp(icon->name, NAME_SYSTEM) == 0)
  {
    CreateSystemFolder(f->base);
    f->atime = -1;
  }
  else if(ISPSYSTEM(icon->type) && strcmp(icon->name, NAME_PATH) == 0)
  {
    CreatePathFolder(f->base);
    f->atime = -1;
  }
  else if(ISPFOLDER(icon->type))
  {
    ScanFolder(f, icon->path, GlobalOptions.HideDotFiles);
    f->atime = GetATime(f->path);
  }

  XtRealizeWidget(f->widget);
  XSetWMProtocols (XtDisplay(root), XtWindow(f->widget), &wm_delete_window, 1);
  n = 0;
  XtSetArg(args[n], XtNwidth, &width); n++;
  XtSetArg(args[n], XtNheight, &height); n++;
  XtGetValues(f->widget, args, n);
  f->width = width;
  f->height = height;
}

void ReleaseFolderItems(folder)
Folder *folder;
{
  int i;
  if(folder->folders != NULL)
  {
    for(i=0;folder->folders[i] != NULL;i++)
    {
      XtFree(folder->folders[i]->name);
      XtFree((char *)folder->folders[i]);
    }
    XtFree((char *)folder->folders);
    folder->folders = NULL;
  }
  if(folder->files != NULL)
  {
    for(i=0;folder->files[i] != NULL;i++)
    {
      XtFree(folder->files[i]->name);
      XtFree((char *)folder->files[i]);
    }
    XtFree((char *)folder->files);
    folder->files = NULL;
  }
}

void RescanFolder(folder)
Folder *folder;
{
  ReleaseFolderItems(folder);
  DestroyFolderIcons(folder);
  if(strcmp(folder->path, TrashFolder) == 0)
  {
    CreateIcon(folder->base, 0, 0, "Destroy", NAME_DESTROY, "",
        GetIconData(ICON_DESTROY), P_SYSTEM);
    ScanFolder(folder, folder->path, 0);
  }
  else
    ScanFolder(folder, folder->path, GlobalOptions.HideDotFiles);
}

void DestroyFolderIcons(folder)
Folder *folder;
{
  int i;
  for (i=0;i<_nicon;i++)
  {
    if (!_icons[i]->deleted && _icons[i]->parent == folder->base)
    {
      DestroyIcon(_icons[i]->widget, NULL, NULL, 0);
    }
  }
}

void CheckFolder(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Folder *folder;
  time_t mtime;
  folder = GetFolder(w);
  if(folder == NULL) return;
  if(folder->atime == -1) return;
dprintf("CheckFolder(%s)\n", folder->path);
  mtime = GetMTime(folder->path);
  if(mtime > folder->atime)
  {
    folder->atime = mtime;
    /*RescanFolder(folder);*//*very slowly!!!*/
  }
}

void DragFolder(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Folder *folder;
  folder = GetFolder(w);
  if(folder == NULL) return;
dprintf("DragFolder(%s)\n", folder->path);
  UnselectIcons();
}

void ReconfigureFolder(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Folder *folder;
  folder = GetFolder(w);
  if(folder == NULL) return;
dprintf("ResizeFolder(%s)\n", folder->path);
  folder->width = e->xconfigure.width;
  folder->height = e->xconfigure.height;
}

void MappedFolder(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Folder *folder;
  folder = GetFolder(w);
  if(folder == NULL) return;
dprintf("MappedFolder(%s)\n", folder->path);
  folder->minimized = False;
}

void IconifyFolder(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Folder *folder;
  folder = GetFolder(w);
  if(folder == NULL) return;
dprintf("IconifyFolder(%s)\n", folder->path);
  if(GlobalOptions.HideWMIcons)
    CloseFolder(w, e, params, num_params);
  else
    folder->minimized = True;
}

void CloseFolder(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Folder *folder;
  folder = GetFolder(w);
  if(folder == NULL) return;
dprintf("CloseFolder(%s)\n", folder->path);
  ReleaseFolderItems(folder);
  DestroyFolderIcons(folder);
  XtUninstallTranslations(folder->widget);
  XtDestroyWidget(folder->base);
  XtDestroyWidget(folder->view);
  XtDestroyWidget(folder->widget);
  /*XtFree(folder->path);*/
  folder->deleted = True;
}

void DestroyFolderAll(path)
char *path;
{
  int i;
  for(i=0;i<_nfolder;i++)
  {
    if (!_folders[i]->deleted && 
        strncmp(_folders[i]->path, path, strlen(path)) == 0 &&
        (_folders[i]->path[strlen(path)] == '\0' ||
         _folders[i]->path[strlen(path)] == '/') )
    {
      CloseFolder(_folders[i]->widget, NULL, NULL, 0);
      *(_folders[i]->path) = '\0';
    }
  }
}

void DropToFolder(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Folder *folder;
  Icon icon;
  folder = GetFolder(w);
  if(folder == NULL) return;
  if(!Dragging) return;
dprintf("DropToFolder(%s)\n", folder->path);
  icon.widget = folder->base;
  icon.type = folder->type;
  icon.name = XtNewString(folder->path);
  icon.path = XtNewString(folder->path);
  icon.deleted = False;
  DropToIcon(&icon, e->xbutton.state);
  XtFree(icon.path);
  XtFree(icon.name);
  folder->atime = GetATime(folder->path);
  RestoreDraggingCursor();
  if(strcmp(folder->path, TrashFolder) == 0)
    RedrawTrashIcons();
}

