/* sxplorer:
 * Copyright 1996 Y.YONEYAMA(fj9y-ynym@asahi-net.or.jp)
 */
#include "sxplorer.h"
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/cursorfont.h>
#include <X11/extensions/shape.h>
#include <X11/Xaw/Simple.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xatom.h>


void ShapeIcon();
void DrawIcon();
void ClickIcon();
void DragIcon();
void DropIcon();
void OpenIcon();
void DestroyIcon();

void EnterPointer();
void LeavePointer();

void DropToIcon();

/* sxplorer.c */
extern Widget root;
extern GC gc;
extern int display_width, display_height;
extern unsigned long fgcolor, bgcolor, cfgcolor, cbgcolor, sfgcolor, sbgcolor;
extern char TrashFolder[];

/* data.c */
extern struct _IconData IconData[MAX_ICONDATA];
extern int _nicondata;

Icon *_icons[MAX_ICONS];
int _nicon = 0;
Boolean Dragging = False;
Icon *DraggingIcon;
Cursor DraggingCursor;
Cursor stdcursor, waitcursor;

void InitIcon()
{
  static XtActionsRec actions[] = {
     {"DrawIcon", DrawIcon},
     {"OpenIcon", OpenIcon},
     {"ClickIcon", ClickIcon},
     {"DragIcon", DragIcon},
     {"DropIcon", DropIcon},
     {"DestroyIcon", DestroyIcon},
  };
  XtAddActions(actions, XtNumber(actions));
  /* create default icon data */
  init_icondata();
  /* create cursor */
  stdcursor = XCreateFontCursor(XtDisplay(root), XC_top_left_arrow);
  waitcursor = XCreateFontCursor(XtDisplay(root), XC_watch);
}

int NewIconNo()
{
  int i;
  for (i=0;i<_nicon;i++)
  {
    if(_icons[i]->deleted)
    {
      XtFree((char *)_icons[i]);
      return(i);
    }
  }
  if(_nicon < MAX_ICONS - 1)
  {
    int i;
    i = _nicon;
    _nicon++;
    return(i);
  }
  return(MAX_ICONS - 1);
}

#define MAX_ARGS 20

Icon *CreateIcon(parent, x, y, name, path, option, picture, type)
Widget parent;
int x, y;
char *name;
char *path;
char *option;
int picture;
p_mode type;
{
  Icon *icon;
  Arg    args[MAX_ARGS];
  int    n;
  static String translations = " \
     <Expose>:                DrawIcon() \n\
     <Destroy>:               DestroyIcon() \n\
     <Btn1Down>(2),<Btn1Up>:  OpenIcon() \n\
     <Btn1Down>:              ClickIcon() \n\
     <Btn1Down>,<Motion>:     DragIcon() \n\
     <Btn1Up>:                DropIcon()";

  if(strncmp(path, "//", 2) == 0) path++;
  icon = XtNew(Icon);
  icon->deleted = False;
  icon->selected = False;
  icon->dragover = False;
  icon->parent = parent;
  icon->type = type;
  if(ISPLINK(type))
  {
    mode_t mode;
    if(ISPALIAS(icon->type))
      icon->type = P_LINK | GetPMode(path) | P_ALIAS;
    else
      icon->type = P_LINK | GetPMode(path);
    mode = GetLMode(path);
    if(mode == NOTEXISTS)
      icon->picture = GetIconData(ICON_BROKEN);
    else if(IsFolder(mode))
      icon->picture = GetIconData(ICON_FLINK);
    else
      icon->picture = GetIconData(ICON_LINK);
  }
  else
    icon->picture = picture;
  icon->x = x;
  icon->y = y;
  icon->name = XtNewString(name);
  icon->path = XtNewString(path);
  icon->option = XtNewString(option);

  if (parent == root)
  {
    n = 0;
    XtSetArg(args[n], XtNx, x); n++;
    XtSetArg(args[n], XtNy, y); n++;
    XtSetArg(args[n], XtNwidth, ICON_WIDTH); n++;
    XtSetArg(args[n], XtNheight, IconData[icon->picture].height); n++;
    XtSetArg(args[n], XtNoverrideRedirect, True); n++;
    XtSetArg(args[n], XtNtranslations, 
        XtParseTranslationTable(translations)); n++;
    icon->widget = XtCreateApplicationShell("icon",
        topLevelShellWidgetClass, args, n);
    n = 0;
    XtSetArg(args[n], XtNoverrideRedirect, True); n++;
    XtSetArg(args[n], XtNx, x); n++;
    /*XtSetArg(args[n], XtNy, y + IconData[icon->picture].height + 4); n++;*/
    XtSetArg(args[n], XtNy, display_height); n++;
    XtSetArg(args[n], XtNborderWidth, 0); n++;
    icon->base = XtCreateApplicationShell(PROGRAM_NAME,
        topLevelShellWidgetClass, args, n);
    n = 0;
    /*XtSetArg(args[n], XtNwidth, ICON_WIDTH); n++;*/
    XtSetArg(args[n], XtNlabel, icon->name); n++;
    XtSetArg(args[n], XtNinternalHeight, 1); n++;
    XtSetArg(args[n], XtNinternalWidth, 1); n++;
    XtSetArg(args[n], XtNforeground, cfgcolor); n++;
    XtSetArg(args[n], XtNbackground, cbgcolor); n++;
    icon->label = XtCreateManagedWidget("caption", labelWidgetClass,
        icon->base, args, n);
  }
  else
  {
    n = 0;
    XtSetArg(args[n], XtNwidth, ICON_WIDTH); n++;
    XtSetArg(args[n], XtNheight, 64); n++;
    XtSetArg(args[n], XtNborderWidth, 0); n++;
    XtSetArg(args[n], XtNforeground, fgcolor); n++;
    XtSetArg(args[n], XtNbackground, bgcolor); n++;
    icon->base = XtCreateManagedWidget("folder", formWidgetClass,
        parent, args, n);
    n = 0;
    XtSetArg(args[n], XtNwidth, ICON_WIDTH); n++;
    XtSetArg(args[n], XtNheight, IconData[icon->picture].height); n++;
    XtSetArg(args[n], XtNtranslations, 
        XtParseTranslationTable(translations)); n++;
    XtSetArg(args[n], XtNborderWidth, 0); n++;
    XtSetArg(args[n], XtNforeground, fgcolor); n++;
    XtSetArg(args[n], XtNbackground, bgcolor); n++;
    XtSetArg(args[n], XtNcursor, stdcursor); n++;
    icon->widget = XtCreateManagedWidget("icon", simpleWidgetClass,
        icon->base, args, n);
    n = 0;
    XtSetArg(args[n], XtNfromVert, icon->widget); n++;
    XtSetArg(args[n], XtNwidth, ICON_WIDTH); n++;
    XtSetArg(args[n], XtNlabel, icon->name); n++;
    XtSetArg(args[n], XtNborderWidth, 0); n++;
    XtSetArg(args[n], XtNinternalHeight, 1); n++;
    XtSetArg(args[n], XtNinternalWidth, 1); n++;
    XtSetArg(args[n], XtNforeground, fgcolor); n++;
    XtSetArg(args[n], XtNbackground, bgcolor); n++;
    icon->label = XtCreateManagedWidget("folder", labelWidgetClass,
        icon->base, args, n);
  }
  XtAddEventHandler(icon->widget, EnterWindowMask|OwnerGrabButtonMask,
      False, EnterPointer, NULL);
  XtAddEventHandler(icon->widget, LeaveWindowMask|OwnerGrabButtonMask,
      False, LeavePointer, NULL);
  if (parent == root)
  {
    XtRealizeWidget(icon->widget);
    XShapeCombineMask(XtDisplay(parent), XtWindow(icon->widget), ShapeBounding,
        (ICON_WIDTH - IconData[icon->picture].width)/2, 0,
        IconData[icon->picture].mask, ShapeSet);
    XDefineCursor(XtDisplay(parent), XtWindow(icon->widget), stdcursor);
    XtRealizeWidget(icon->base);
  }
  _icons[NewIconNo()] = icon;
  return(icon);
}


Icon *GetIcon(w)
Widget w;
{
  int i;
  for (i=0;i<_nicon;i++)
  {
    if (!_icons[i]->deleted && (_icons[i]->widget == w ||
        _icons[i]->label == w))
      return(_icons[i]);
  }
  return(NULL);
}

Icon *GetIconFromWindow(w)
Window w;
{
  int i;
  for (i=0;i<_nicon;i++)
  {
    if (!_icons[i]->deleted && (XtWindow(_icons[i]->widget) == w ||
        XtWindow(_icons[i]->label) == w))
      return(_icons[i]);
  }
  return(NULL);
}

Boolean IsTrashFolder(icon)
Icon *icon;
{
  if(icon->deleted == False && ISPSYSTEM(icon->type) && 
     strcmp(icon->path, TrashFolder) == 0)
    return(True);
  return(False);
}

Icon *GetTrashFolder()
{
  int i;
  for (i=0;i<_nicon;i++)
  {
    if (IsTrashFolder(_icons[i]))
      return(_icons[i]);
  }
  return(NULL);
}

Icon *GetIconFromPath(path)
char *path;
{
  int i;
  for (i=0;i<_nicon;i++)
  {
    if (!_icons[i]->deleted && strcmp(_icons[i]->path, path) == 0)
      return(_icons[i]);
  }
  return(NULL);
}


Boolean IsAcceptedIcon(icon)
Icon *icon;
{
  if(ISPFOLDER(icon->type))
  {
    if(ISPWRITE(icon->type))
      return(True);
    else
      return(False);
  }
  if(ISPEXEC(icon->type))
    return(True);
  return(False);
}

void DrawIcon(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Icon *icon;
  Arg args[10];
  int n;

  if((icon = GetIcon(w)) == NULL)
    return;
  if(icon->parent == root)
  {
    Window ir, cr;
    int ix, iy, cx, cy;
    unsigned int iw, ih, ib, id, cw, ch, cb, cd;
    XGetGeometry(XtDisplay(icon->widget), XtWindow(icon->widget), &ir,
        &ix, &iy, &iw, &ih, &ib, &id);
    XGetGeometry(XtDisplay(icon->base), XtWindow(icon->base), &cr,
        &cx, &cy, &cw, &ch, &cb, &cd);
    XMoveWindow(XtDisplay(icon->base), XtWindow(icon->base),
        ix + (iw - cw)/2, iy + IconData[icon->picture].height + 4);
  }
  if(IsTrashFolder(icon))
  {
    int oldpic = icon->picture;
    if (CountFolderItems(TrashFolder) == 0)
      icon->picture = GetIconData(ICON_TRASH);
    else
      icon->picture = GetIconData(ICON_TRASHFULL);
    if(icon->picture != oldpic)
    {
      XShapeCombineMask(XtDisplay(icon->widget), XtWindow(icon->widget),
        ShapeBounding, (ICON_WIDTH - IconData[icon->picture].width)/2, 0,
        IconData[icon->picture].mask, ShapeSet);
    }
  }
  if (icon->selected)
  {
    XCopyArea(XtDisplay(w), IconData[icon->picture].open_picture,
        XtWindow(w), gc, 0, 0,
        IconData[icon->picture].width, IconData[icon->picture].height,
        (ICON_WIDTH - IconData[icon->picture].width)/2, 0);
    n = 0;
    XtSetArg(args[n], XtNbackground, sbgcolor); n++;
    XtSetValues(icon->label, args, n);
    n = 0;
    XtSetArg(args[n], XtNforeground, sfgcolor); n++;
    XtSetValues(icon->label, args, n);
    return;
  }
  else
  {
    XCopyArea(XtDisplay(w), IconData[icon->picture].picture,
        XtWindow(w), gc, 0, 0,
        IconData[icon->picture].width, IconData[icon->picture].height,
        (ICON_WIDTH - IconData[icon->picture].width)/2, 0);
  }
  if (icon->dragover && IsAcceptedIcon(icon))
  {
    n = 0;
    XtSetArg(args[n], XtNbackground, fgcolor); n++;
    XtSetArg(args[n], XtNforeground, bgcolor); n++;
    XtSetValues(icon->label, args, n);
  }
  else
  {
    n = 0;
    if(icon->parent == root)
    {
      XtSetArg(args[n], XtNbackground, cbgcolor); n++;
      XtSetArg(args[n], XtNforeground, cfgcolor); n++;
    }
    else
    {
      XtSetArg(args[n], XtNbackground, bgcolor); n++;
      XtSetArg(args[n], XtNforeground, fgcolor); n++;
    }
    XtSetValues(icon->label, args, n);
  }
}

void RedrawTrashIcons()
{
  int i;
  for(i=0;i<_nicon;i++)
    if(IsTrashFolder(_icons[i]))
      DrawIcon(_icons[i]->widget, NULL, NULL, 0);
}

void UnselectIcons()
{
  int i;
  for(i=0;i<_nicon;i++)
  {
    if(!_icons[i]->deleted && _icons[i]->selected)
    {
      _icons[i]->selected = False;
      DrawIcon(_icons[i]->widget, NULL, NULL, 0);
    }
  }
}

void RestoreDraggingCursor()
{
  Widget w;
  if(!Dragging)
      return;
  w = DraggingIcon->widget;
  XUndefineCursor(XtDisplay(w), XtWindow(w));
  XDefineCursor(XtDisplay(w), XtWindow(w), stdcursor);
  XFreeCursor(XtDisplay(w), DraggingCursor);
  Dragging = False;
}

#ifdef ENABLE_SELECTION
Boolean convertSelection(w, sel, targ, type, value, len, fmt)
Widget w;
Atom *sel, *targ, *type;
caddr_t *value;
unsigned long *len;
int *fmt;
{
  Icon *icon;
  if((icon = GetIcon(w)) == NULL)
    return False;
dprintf("convertSelection(%s)\n", icon->path);
  if(*targ != XA_STRING) return False;
  *value = XtNewString(icon->path);
  *len = strlen(icon->path);
  *fmt=8;
  return True;
}
#endif /* ENABLE_SELECTION */


void ClickIcon(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{ 
  Icon *icon;
  if((icon = GetIcon(w)) == NULL)
    return;
dprintf("ClickIcon(%s)\n", icon->name);
  if((e->xbutton.state & ControlMask) == ControlMask)
  {
    icon->selected = !icon->selected;
  }
  else
  {
    if (!icon->selected)
      UnselectIcons();
    icon->selected = True;
  }
  DrawIcon(icon->widget, NULL, NULL, 0);
#ifdef ENABLE_SELECTION
  /* copy to clipboard */
  XtOwnSelection(w, XA_PRIMARY, e->xbutton.time,
      convertSelection, NULL, NULL);
#endif /* ENABLE_SELECTION */
}

void DragIcon(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Cursor cs;
  XColor fg, bg;
  Colormap cm;
  Icon *icon;
  int i, selects = 0;
  if((icon = GetIcon(w)) == NULL)
    return;
dprintf("DragIcon(%s)\n", icon->name);
  fg.pixel = BlackPixel(XtDisplay(w), 0);
  bg.pixel = WhitePixel(XtDisplay(w), 0);
  cm = DefaultColormap(XtDisplay(w), 0);
  XQueryColor(XtDisplay(w), cm, &fg);
  XQueryColor(XtDisplay(w), cm, &bg);
  icon->selected = True;
  for(i=0;i<_nicon;i++)
  {
    if(_icons[i]->deleted == False && _icons[i]->selected == True) selects++;
    if(selects > 1) break;
  }
  if(selects == 1)
  {
    cs = XCreatePixmapCursor(XtDisplay(icon->widget),
        IconData[icon->picture].drag_cursor,
        IconData[icon->picture].mask, &fg, &bg,
        IconData[icon->picture].width/2, IconData[icon->picture].height/2);
  }
  else if(selects >= 2)
  {
    int picture;
    picture = GetIconData(ICON_MANYTEXT);
    cs = XCreatePixmapCursor(XtDisplay(icon->widget),
        IconData[picture].drag_cursor,
        IconData[picture].mask, &fg, &bg,
        IconData[picture].width/2, IconData[picture].height/2);
  }
  XUndefineCursor(XtDisplay(icon->widget), XtWindow(icon->widget));
  XDefineCursor(XtDisplay(icon->widget), XtWindow(icon->widget), cs);
  Dragging = True;
  DraggingIcon = icon;
  DraggingCursor = cs;
  DrawIcon(icon->widget, NULL, NULL, 0);
}

void DropIcon(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Icon *icon;

  Window root_r, child_r;
  int rx_r, ry_r, wx_r, wy_r;
  unsigned int mask_r;

  if(!Dragging)
     return;

  RestoreDraggingCursor();
  Dragging = False;

  if((icon = GetIcon(w)) == NULL)
    return;

  if(icon != DraggingIcon)
  {
    icon->dragover = False;
    DrawIcon(icon->widget, NULL, NULL, 0);
  }
dprintf("DropIcon(%s)\n", icon->name);

  if(icon == DraggingIcon) /* Drop to not icon (drop to self)*/
  {
    XQueryPointer(XtDisplay(w), RootWindow(XtDisplay(root), 0), &root_r,
        &child_r, &rx_r, &ry_r, &wx_r, &wy_r, &mask_r);
    if(icon->parent == root && (e->xbutton.state & ShiftMask) == 0)
    {  /* move alias */
      XMoveWindow(XtDisplay(w), XtWindow(icon->widget),
           rx_r - ICON_WIDTH/2, ry_r - IconData[icon->picture].height/2);
      icon->x = rx_r - ICON_WIDTH/2;
      icon->y = ry_r - IconData[icon->picture].height/2;
      DrawIcon(icon->widget, NULL, NULL, 0);
    }
    else /* create new alias */
    {
      if(child_r == NULL)
      {
        Icon *new;
        char name[256];
        strncpy(name, icon->name, 255);
        if((e->xbutton.state & ShiftMask) == 0 &&
           (name[0] >= 'a' && name[0] <= 'z'))
            name[0] -= 0x20;
        new = CreateIcon(root, rx_r - ICON_WIDTH/2,
            ry_r - IconData[icon->picture].height/2, name,
            icon->path, icon->option, icon->picture, icon->type | P_ALIAS);
        UnselectIcons();
        new->selected = True;
        DrawIcon(new->widget, NULL, NULL, 0);
      }
    }
  }
  else /* drop to other icon */
  {
    DropToIcon(icon, e->xbutton.state);
  }
  Dragging = False;
}

void OpenIcon(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal num_params;
{
  Icon *icon;
  if((icon = GetIcon(w)) == NULL)
    return;
dprintf("OpenIcon(%s)\n", icon->name);

  XUndefineCursor(XtDisplay(icon->widget), XtWindow(icon->widget));
  XDefineCursor(XtDisplay(icon->widget), XtWindow(icon->widget), waitcursor);
  XFlush(XtDisplay(icon->widget));

  if (ISPFOLDER(icon->type) || ISPSYSTEM(icon->type))
  {
    if(ISPFOLDER(icon->type) && ISPREAD(icon->type))
      OpenFolder(icon);
    else if(ISPSYSTEM(icon->type))
      OpenSystem(icon);
    else
    {
      PermitDialog();
      goto ret;
    }

    if((e->xbutton.state & ShiftMask) == ShiftMask &&
       icon->parent != root)
    {
      Folder *f, *GetFolder();
      f = GetFolder(icon->parent);
      if(f != NULL)
        CloseFolder(f->widget, NULL, NULL, 0);
    }
  }
  else
  {
    ExecIcon(icon);
    sleep(1);
  }
ret:
  UnselectIcons();
  DrawIcon(icon->widget, NULL, NULL, 0);

  XUndefineCursor(XtDisplay(icon->widget), XtWindow(icon->widget));
  XDefineCursor(XtDisplay(icon->widget), XtWindow(icon->widget), stdcursor);
}


void EnterPointer(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Icon *icon;
  if(!Dragging) return;
  if((icon = GetIcon(w)) == NULL)
    return;

dprintf("EnterPointer(%s)\n", icon->name);
  if(!icon->selected)
    icon->dragover = True;
  DrawIcon(icon->widget, NULL, NULL, 0);
}

void LeavePointer(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Icon *icon;
  if(!Dragging) return;
  if((icon = GetIcon(w)) == NULL)
    return;
dprintf("LeavePointer(%s)\n", icon->name);
  icon->dragover = False;
  DrawIcon(icon->widget, NULL, NULL, 0);
}
  
void DestroyIcon(w, e, params, num_params)
Widget w;
XEvent *e;
String *params;
Cardinal *num_params;
{
  Icon *icon;
  if((icon = GetIcon(w)) == NULL)
    return;
dprintf("DestoryIcon(%s)\n", icon->name);
  XtFree(icon->name);
  XtFree(icon->path);
  XtFree(icon->option);
  XtRemoveEventHandler(icon->widget, EnterWindowMask|OwnerGrabButtonMask,
      False, EnterPointer, NULL);
  XtRemoveEventHandler(icon->widget, LeaveWindowMask|OwnerGrabButtonMask,
      False, LeavePointer, NULL);
  if(icon->parent == root)
    XUndefineCursor(XtDisplay(icon->widget), XtWindow(icon->widget));
  XtDestroyWidget(icon->widget);
  XtDestroyWidget(icon->label);
  XtDestroyWidget(icon->base);
  icon->deleted = True;
}

void DestroyIconAll(path)
char *path;
{
  int i;
  for (i=0;i<_nicon;i++)
  {
    if (!_icons[i]->deleted &&
         strcmp(_icons[i]->path, TrashFolder) != 0 &&
         strncmp(_icons[i]->path, path, strlen(path)) == 0 &&
         (_icons[i]->path[strlen(path)] == '\0' || 
          _icons[i]->path[strlen(path)] == '/') )
      DestroyIcon(_icons[i]->widget, NULL, NULL, 0);
  }
}

void CopyFunc(from, to, icon, folder)
char *from;
char *to;
Icon *icon;
Icon *folder;
{
  mode_t mode;
  Folder *f;
  Folder *IsOpenedFolder();

  mode = GetMode(to);
  if(CopyFile(from, to))
  {
    if((f = IsOpenedFolder(folder->path)) != NULL && mode == NOTEXISTS)
    {
      CreateIcon(f->base, 0, 0, icon->name, to, icon->option, icon->picture,
        icon->type & ~P_ALIAS);
    }
  }
  else
  {
    XBell(XtDisplay(icon->widget), 0);
  }
}

void CopyIconToFolder(icon, folder)
Icon *icon;
Icon *folder;
{
  char path[1024];
  char *GetFileName();
  void GetRealPath();

  GetRealPath(path, folder->path);
  strcat(path, "/");
  strcat(path, GetFileName(icon->path));
  if(strcmp(icon->path, path) == 0)
    return;
  if(access(path, F_OK) == 0)
  {
    if(access(path, W_OK) == 0)
      OverwriteDialog(icon->path, path, icon, folder, CopyFunc);
    else
      PermitDialog();
  }
  else
  {
    if(access(folder->path, W_OK|X_OK) == 0)
      CopyFunc(icon->path, path, icon, folder);
    else
      PermitDialog();
  }
}

void MoveFunc(from, to, icon, folder)
char *from;
char *to;
Icon *icon;
Icon *folder;
{
  Folder *f;
  mode_t mode;
  Folder *IsOpenedFolder();

  mode = GetMode(to);
  if(MoveFile(from, to))
  {
    if(ISPFOLDER(icon->type))
      DestroyFolderAll(icon->path);
    if((f = IsOpenedFolder(folder->path)) != NULL && mode == NOTEXISTS)
    {
      CreateIcon(f->base, 0, 0, icon->name, to, icon->option, icon->picture,
        icon->type & ~P_ALIAS);
    }
    DestroyIconAll(icon->path);
    /* redraw TrashCan */
    if(strncmp(icon->path, TrashFolder, strlen(TrashFolder)) == 0 &&
       icon->path[strlen(TrashFolder)] == '/')
      RedrawTrashIcons();
  }
  else
    XBell(XtDisplay(icon->widget), 0);
}

void MoveIconToFolder(icon, folder)
Icon *icon;
Icon *folder;
{
  char fromdir[1024];
  char path[1024];
  char *GetFileName(), *GetDirectory();
  void GetRealPath();

  GetRealPath(path, folder->path);
  strcat(path, "/");
  strcat(path, GetFileName(icon->path));

  if(strcmp(icon->path, path) == 0)
  {
    if(ISPALIAS(icon->type))
      DestroyIcon(icon->widget, NULL, NULL, 0);
    return;
  }
  if(access(GetDirectory(fromdir, icon->path), W_OK) != 0)
  {
      PermitDialog();
      return;
  }
  if(access(path, F_OK) == 0)
  {
    if(access(path, W_OK) == 0)
      OverwriteDialog(icon->path, path, icon, folder, MoveFunc);
    else
      PermitDialog();
  }
  else
  {
    if(access(folder->path, W_OK|X_OK) == 0)
      MoveFunc(icon->path, path, icon, folder);
    else
      PermitDialog();
  }
}

void DropToIcon(icon, state)
Icon *icon;
unsigned int state;
{
  int i;

dprintf("DropToIcon(%s)\n", icon->name);

  if (IsTrashFolder(icon))
  {
    for(i=0;i<_nicon;i++)
    {
      if(!_icons[i]->deleted && _icons[i]->selected)
      { /* remove icon */
        if(ISPALIAS(_icons[i]->type)) /* remove alias */
          RemoveAliasDialog(_icons[i]);
        else if(ISPSYSTEM(_icons[i]->type))
          DestroySystem(_icons[i]);
        else /* remove file (move to TrashCan) */
          MoveIconToFolder(_icons[i], icon);
      }
    }
    RedrawTrashIcons();
    return;
  }
  else if(ISPFOLDER(icon->type))
  {
    for(i=0;i<_nicon;i++)
    {
      if(!_icons[i]->deleted && _icons[i]->selected)
      { /* move icon */
        if(ISPSYSTEM(_icons[i]->type))
          XBell(XtDisplay(icon->widget), 0);
        else
        {
          if((state & ShiftMask) == ShiftMask)
            CopyIconToFolder(_icons[i], icon);
          else
            MoveIconToFolder(_icons[i], icon);
        }
      }
    }
  }
  else if(ISPSYSTEM(icon->type))
  {
    ExecSystem(icon);
  }
  else if(ISPEXEC(icon->type))
  {
    ExecIcon(icon);
  }
  else
  {
    XBell(XtDisplay(icon->widget), 0);
    return;
  }
  UnselectIcons();
}

