/* sxplorer:
 * Copyright 1996 Y.YONEYAMA(fj9y-ynym@asahi-net.or.jp)
 */
#include "sxplorer.h"
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <signal.h>


Widget root = NULL;
Display *display;
int display_width, display_height;
GC gc;
Atom wm_delete_window;
unsigned long fgcolor, bgcolor, cfgcolor, cbgcolor, sfgcolor, sbgcolor;
char HomeFolder[1024];
char DesktopFile[1024];
char TrashFolder[1024];
Boolean TmpTrash = False;

struct global_options GlobalOptions;

typedef struct {
  String desktopfile;
  Pixel fore_color;
  Pixel back_color;
  Pixel cfore_color;
  Pixel cback_color;
  Pixel sfore_color;
  Pixel sback_color;
} AppData;
AppData app_data;

#define XtNdesktopFile "desktopFile"
#define XtCdesktopFile "DesktopFile"
#define XtNcapForeground "capForeground"
#define XtNcapBackground "capBackground"
#define XtNselForeground "selForeground"
#define XtNselBackground "selBackground"

static XtResource resources[] = {
  {XtNdesktopFile, XtCdesktopFile, XtRString, sizeof(String),
   XtOffsetOf(AppData,desktopfile), XtRString, DESKTOP_FILE},
  {XtNforeground, XtCForeground, XtRPixel, sizeof (Pixel),
   XtOffsetOf(AppData,fore_color), XtRString, XtDefaultForeground},
  {XtNbackground, XtCBackground, XtRPixel, sizeof (Pixel),
   XtOffsetOf(AppData,back_color), XtRString, XtDefaultBackground},
  {XtNcapForeground, XtCForeground, XtRPixel, sizeof (Pixel),
   XtOffsetOf(AppData,cfore_color), XtRString, XtDefaultForeground},
  {XtNcapBackground, XtCBackground, XtRPixel, sizeof (Pixel),
   XtOffsetOf(AppData,cback_color), XtRString, XtDefaultBackground},
  {XtNselForeground, XtCForeground, XtRPixel, sizeof (Pixel),
   XtOffsetOf(AppData,sfore_color), XtRString, XtDefaultForeground},
  {XtNselBackground, XtCBackground, XtRPixel, sizeof (Pixel),
   XtOffsetOf(AppData,sback_color), XtRString, "Gray"},
};

XrmOptionDescRec options[] = {
  {"-desktopfile", "*desktopFile", XrmoptionSepArg, NULL},
};

unsigned long GetColorPixel(color)
String color;
{
  Colormap cmap;
  XColor c0, c1;
  cmap = DefaultColormap(XtDisplay(root), 0);
  XAllocNamedColor(XtDisplay(root), cmap, color, &c0, &c1);
  return(c0.pixel);
}

void quit()
{
  WriteAliases();
  if(TmpTrash)
    RemoveFile(TrashFolder);
  exit(0);
}

int ErrorHandler(dpy)
Display *dpy;
{
  perror("Xlib Fatal Error");
  quit();
}

#ifndef SYSV
void sig_child()
{
  int status;
  wait(&status);
}
#endif

void main(argc,argv)
int argc;
char **argv;
{
  void quit();
  mode_t mode;
  p_mode GetPMode();

  signal(SIGHUP, quit);
#ifndef SYSV
  signal(SIGCHLD, sig_child);
#else
  signal(SIGCHLD, SIG_IGN);
#endif

  root = XtInitialize(argv[0], "SXPlorer", options, XtNumber(options),
      &argc, argv);

  XtGetApplicationResources(root, &app_data, resources, XtNumber(resources),
     NULL, 0);

  XSetIOErrorHandler(ErrorHandler);

  wm_delete_window = XInternAtom(XtDisplay(root), "WM_DELETE_WINDOW", False);

  display = XtDisplay(root);
  gc = XCreateGC(XtDisplay(root), RootWindow(XtDisplay(root), 0), 0, 0);
  fgcolor = app_data.fore_color;
  bgcolor = app_data.back_color;
  cfgcolor = app_data.cfore_color;
  cbgcolor = app_data.cback_color;
  sfgcolor = app_data.sfore_color;
  sbgcolor = app_data.sback_color;
  display_width = DisplayWidth(XtDisplay(root),0);
  display_height = DisplayHeight(XtDisplay(root),0);

  sprintf(HomeFolder, "%s", (char *)getenv("HOME"));
  sprintf(DesktopFile, "%s/%s", HomeFolder, app_data.desktopfile);
  sprintf(TrashFolder, "%s/%s", HomeFolder, TRASH_FOLDER);
  mode = GetMode(TrashFolder);
  if(mode == NOTEXISTS)
  {
    if(MakeTrashFolder(TrashFolder) == False)
    {
      char TmpFolder[256];
      sprintf(TmpFolder, "/tmp/.sxplorer.trash.%d", (int)getpid());
      if(MakeTrashFolder(TmpFolder) == False)
      {
        printf("%s: cannot create trash folder(%s or %s).\n",
            PROGRAM_NAME, TrashFolder, TmpFolder);
        exit(1);
      }
      strcpy(TrashFolder, TmpFolder);
      TmpTrash = True;
    }
  }
  GlobalOptions.HideDotFiles = False;
  GlobalOptions.HideWMIcons = False;

  InitIcon();
  InitFolder();

  LoadIniFile();

  if(ReadAliases() == 0)
  {
    CreateIcon(root, display_width - ICON_WIDTH - 16, 16,
      "System", NAME_SYSTEM, "", GetIconData(ICON_SYSTEM), P_SYSTEM);
    CreateIcon(root, display_width - ICON_WIDTH - 16, 96,
      "TrashCan", TrashFolder, "", GetIconData(ICON_TRASH),
      P_SYSTEM | GetPMode(TrashFolder));
  }

  XtMainLoop();
}

