/* sxplorer:
 * Copyright 1996 Y.YONEYAMA(fj9y-ynym@asahi-net.or.jp)
 */

#define PROGRAM_NAME   "sxplorer"
#define VERSION        "1.00"

#include <X11/Intrinsic.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "config.h"

#ifdef DEBUG
#define dprintf(x,y) printf(x,y)
#else
#define dprintf(x,y)
#endif

typedef int p_mode;

struct global_options {
  Boolean HideDotFiles;
  Boolean HideWMIcons;
};

typedef struct {
  char    *name;
  mode_t  mode;
  uid_t   uid;
  gid_t   gid;
  time_t  mtime;
  long    size;
} FileItem;

struct sFolder {
  Widget widget;
  Widget view;
  Widget base;
  char *path;
  int  type;
  int width;
  int height;
  Boolean deleted;
  Boolean minimized;
  time_t atime;
  FileItem **folders;
  FileItem **files;
};

typedef struct sFolder Folder;

struct sIcon {
  Widget parent;
  Widget base;
  Widget widget;
  Widget label;
  int x;
  int y;
  int labelwidth;
  char *name;
  char *path;
  char *option;
  mode_t mode;
  p_mode  type;
  int  picture;
  Boolean deleted;
  Boolean selected;
  Boolean dragover;
};

typedef struct sIcon Icon;

#define MAX_ICONDATA 256 /* maximum number of icon data */

struct _IconData {
  char *name;
  Pixmap picture;        /* color pixmap */
  Pixmap open_picture;   /* color pixmap */
  Pixmap mask;           /* monochrome bitmap (mask) */
  Pixmap drag_cursor;    /* monochrome bitmap */
  int width;
  int height;
};

#define ICON_WIDTH 72
#define MAX_ICONS 2048

/* default icon key (max len = 20 byte) */
#define ICON_BROKEN    "Broken"
#define ICON_SYSTEM    "System"
#define ICON_TRASH     "Trash"
#define ICON_TRASHFULL "TrashFull"
#define ICON_DESTROY   "Destroy"
#define ICON_EXIT      "Exit"
#define ICON_FOLDER    "Folder"
#define ICON_TEXT      "Text"
#define ICON_MANYTEXT  "ManyText"
#define ICON_EXEC      "Exec"
#define ICON_LINK      "Link"
#define ICON_FLINK     "FLink"

#define ICON_CSOURCE   "Csource"
#define ICON_CHEADER   "Cheader"
#define ICON_MAKE      "Make"
#define ICON_TEX       "TeX"
#define ICON_PICTURE   "Picture"

/* system name (system icons)*/
#define NAME_SYSTEM   "System"
#define NAME_TRASH    "TrashCan"
#define NAME_PATH     "Path"
#define NAME_EXIT     "Exit"
#define NAME_DESTROY  "Destroy"

#define DEFAULT_FILEMODE   0644
#define DEFAULT_TRASHMODE  0700
#define DEFAULT_FOLDERMODE 0755

#define NOTEXISTS  ((mode_t)-1)

#define P_READ           (1 << 0)
#define P_WRITE          (1 << 1)
#define P_EXEC           (1 << 2)
#define P_FOLDER         (1 << 3)
#define P_LINK           (1 << 4)
#define P_ALIAS          (1 << 5)
#define P_SYSTEM         (1 << 6)

#define ISPREAD(x)       (((x)&P_READ)==P_READ)
#define ISPWRITE(x)      (((x)&P_WRITE)==P_WRITE)
#define ISPEXEC(x)       (((x)&P_EXEC)==P_EXEC)
#define ISPFOLDER(x)     (((x)&P_FOLDER)==P_FOLDER)
#define ISPLINK(x)       (((x)&P_LINK)==P_LINK)
#define ISPALIAS(x)      (((x)&P_ALIAS)==P_ALIAS)
#define ISPSYSTEM(x)     (((x)&P_SYSTEM)==P_SYSTEM)

