/* sxplorer:
 * Copyright 1996 Y.YONEYAMA(fj9y-ynym@asahi-net.or.jp)
 */
#include <stdio.h>
#include <X11/Intrinsic.h>

/* sxplorer.c */
extern unsigned long bgcolor;

int GetXpmSize(width,height,colors,num,data)
int *width, *height;
int *colors, *num;
char *data; /* input */
{
  sscanf(data,"%d%d%d%d",width,height,colors,num);
  return(0);
}

static int hex4toi(hex)
char *hex;
{
  int val;
  char tmp[5];

  strncpy(tmp,hex,4);
  tmp[4] = '\0';
  sscanf(tmp,"%04X",&val);
  return(val);
}

static int GetXpmColor(dpy,cmap,col,data)
Display *dpy;
Colormap cmap;
XColor *col;
char *data;
{
  XColor ecol;
  char tmpcol[50];
  int i;
  int s = 0;

  if(*data == '#')
  {
    col->red = hex4toi(data+1);
    col->green = hex4toi(data+5);
    col->blue = hex4toi(data+9);
    XAllocColor(dpy, cmap, col);
  }
  else
  {
    for(i=0;i<50;i++,data++)
    {
      if(*data == '"')
      {
        tmpcol[i] = '\0';
        break;
      }
      tmpcol[i] = *data;
    }
    if(strcmp(tmpcol,"none")==0 || strcmp(tmpcol,"None")==0)
    {
      col->pixel = bgcolor;
      XQueryColor(dpy, cmap, col);
      s = 1;
    }
    else
      XAllocNamedColor(dpy, cmap, tmpcol, col, &ecol);
  }
  return(s);
}

static int GetXpmDarkColor(dpy, cmap, col, data)
Display *dpy;
Colormap cmap;
XColor *col;
char *data;
{
  XColor ecol;
  char tmpcol[50];
  int i;
  int s = 0;

  if(*data == '#')
  {
    col->red = hex4toi(data+1)/2;
    col->green = hex4toi(data+5)/2;
    col->blue = hex4toi(data+9)/2;
    XAllocColor(dpy, cmap, col);
  }
  else
  {
    for(i=0;i<50;i++,data++)
    {
      if(*data == '"')
      {
        tmpcol[i] = '\0';
        break;
      }
      tmpcol[i] = *data;
    }
    if(strcmp(tmpcol,"none")==0 || strcmp(tmpcol,"None")==0)
    {
      col->pixel = bgcolor;
      XQueryColor(dpy, cmap, col);
      s = 1;
    }
    else
    {
      XColor rcol;
      XAllocNamedColor(dpy, cmap, tmpcol, &rcol, &ecol);
      col->red = rcol.red / 2;
      col->green = rcol.green / 2;
      col->blue = rcol.blue / 2;
      XAllocColor(dpy, cmap, col);
    }
  }
  return(s);
}

int getstrlen(str)
char *str;
{
  int i;
  for(i=strlen(str)-1;i>0;i--)
    if(str[i] == '"') break;
  return(i);
}

char **CreatePixmapDataFromFile(dpy, gc, fname)
Display *dpy;
GC gc;
char *fname;
{
  FILE *fp;
  char buf[256];
  char **data;
  int i;
  int width, height, colors, num;

  fp = fopen(fname,"r");
  if(fp == NULL)
    return(NULL);
  while(1)
  {
    if(fgets(buf, 256, fp) == NULL)
    {
      fclose(fp);
      return(NULL);
    }
    if(buf[0] == '"') break;
  }
  GetXpmSize(&width,&height,&colors,&num,&buf[1]);
  data = (char **)XtMalloc(sizeof(char *)*(colors+height+1));
  data[0] = XtMalloc(getstrlen(buf)+1);
  strncpy(data[0], &buf[1], getstrlen(buf));
  data[0][getstrlen(buf)] = '\0';
  for(i=1;i<=colors+height;)
  {
    if(fgets(buf, 256, fp) == NULL)
    {
      fclose(fp);
      return(NULL);
    }
    if(buf[0] != '"') continue;
    data[i] = XtMalloc(getstrlen(buf)+1);
    strncpy(data[i], &buf[1], getstrlen(buf));
    data[i][getstrlen(buf)] = '\0';
    i++;
  }
  fclose(fp);
  return(data);
}

Pixmap _CreatePixmapFromData(dpy, gc, data, state)
Display *dpy;
GC gc;
char **data;
int state;
{
  Pixmap pm;
  Colormap cmap;
  int width, height, colors, num;
  XColor *col;
  char *colc;
  int i,j,k;

  GetXpmSize(&width,&height,&colors,&num,data[0]);
  cmap = DefaultColormap(dpy, 0);
  col = (XColor *)XtMalloc(sizeof(XColor)*colors);
  colc = (char *)XtMalloc(colors);
  for(i=0;i<colors;i++)
  {
    if(state == 1)
      GetXpmDarkColor(dpy,cmap,&col[i],strchr(&data[i+1][2],'c')+2);
    else
      GetXpmColor(dpy,cmap,&col[i],strchr(&data[i+1][2],'c')+2);
    colc[i] = data[i+1][0];
  }
  pm = XCreatePixmap(dpy, RootWindow(dpy, 0), width, height,
         DefaultDepth(dpy, 0));
  XSetFunction(dpy, gc, GXcopy);
  for(i=0;i<height;i++)
  {
    for(j=0;j<width;j++)
    {
      for(k=0;k<colors;k++)
      {
        if(data[i+colors+1][j] == colc[k])
        {
          XSetForeground(dpy, gc, col[k].pixel);
          break;
        }
      }
      XDrawPoint(dpy, pm, gc, j, i);
    }
  }

  XtFree((char *)col);
  XtFree(colc);
  return(pm);
}

Pixmap CreatePixmapFromData(dpy, gc, data)
Display *dpy;
GC gc;
char **data;
{
  return(_CreatePixmapFromData(dpy, gc, data, 0));
}

Pixmap CreateDarkPixmapFromData(dpy, gc, data)
Display *dpy;
GC gc;
char **data;
{
  return(_CreatePixmapFromData(dpy, gc, data, 1));
}

Pixmap CreateBitmapFromPixmapData(dpy, gc, data)
Display *dpy;
GC gc;
char **data;
{
  Pixmap pm;
  Colormap cmap;
  int width, height, colors, num;
  XColor *col;
  char *colc;
  int i,j,k;
  char *bit;
  unsigned int acol;

  GetXpmSize(&width,&height,&colors,&num,data[0]);
  cmap = DefaultColormap(dpy, 0);
  col = (XColor *)XtMalloc(sizeof(XColor)*colors);
  colc = (char *)XtMalloc(colors);
  acol = 0;
  for(i=0;i<colors;i++)
  {
    GetXpmColor(dpy,cmap,&col[i],strchr(&data[i+1][2],'c')+2);
    colc[i] = data[i+1][0];
    acol += (col[i].red+col[i].green+col[i].blue)/3;
  }
  acol /= colors*2;
  bit = (char *)XtCalloc((int)((width+7)/8)*height,1);
  for(i=0;i<height;i++)
  {
    for(j=0;j<width;j++)
    {
      for(k=0;k<colors;k++)
      {
        if(data[i+colors+1][j] == colc[k])
        {
          unsigned int v;
          v = (col[k].red+col[k].green+col[k].blue)/3;
          if(v<acol)
            bit[(int)((width+7)/8)*i+j/8] |= 1<<(j%8);
          break;
        }
      }
    }
  }
  pm = XCreateBitmapFromData(dpy, RootWindow(dpy, 0), bit, width, height);

  XtFree(bit);
  XtFree((char *)col);
  XtFree((char *)colc);
  return(pm);
}

Pixmap CreateMaskFromPixmapData(dpy, gc, data)
Display *dpy;
GC gc;
char **data;
{
  Pixmap pm;
  Colormap cmap;
  int width, height, colors, num;
  XColor col;
  int i,j;
  char *bit;
  char none;

  GetXpmSize(&width,&height,&colors,&num,data[0]);
  cmap = DefaultColormap(dpy, 0);
  for(i=0;i<colors;i++)
  {
    if(GetXpmColor(dpy,cmap,&col,strchr(&data[i+1][2],'c')+2))
      none = data[i+1][0];
  }
  bit = (char *)XtCalloc((int)((width+7)/8)*height,1);
  for(i=0;i<height;i++)
  {
    for(j=0;j<width;j++)
    {
      if(data[i+colors+1][j] != none)
        bit[(int)((width+7)/8)*i+j/8] |= 1<<(j%8);
    }
  }
  pm = XCreateBitmapFromData(dpy, RootWindow(dpy, 0), bit, width, height);

  XtFree(bit);
  return(pm);
}

Pixmap CreatePixmapFromFile(dpy, gc, fname)
Display *dpy;
GC gc;
char *fname;
{
  char **data;
  data = CreatePixmapDataFromFile(dpy, gc, fname);
  if(data == NULL) return(-1);
  return(_CreatePixmapFromData(dpy, gc, data, 0));
}

Pixmap CreateDarkPixmapFromFile(dpy, gc, fname)
Display *dpy;
GC gc;
char *fname;
{
  char **data;
  data = CreatePixmapDataFromFile(dpy, gc, fname);
  if(data == NULL) return(-1);
  return(_CreatePixmapFromData(dpy, gc, data, 1));
}

Pixmap CreateBitmapFromPixmapFile(dpy, gc, fname)
Display *dpy;
GC gc;
char *fname;
{
  char **data;
  data = CreatePixmapDataFromFile(dpy, gc, fname);
  return(CreateBitmapFromPixmapData(dpy, gc, data));
}

Pixmap CreateMaskFromPixmapFile(dpy, gc, fname)
Display *dpy;
GC gc;
char *fname;
{
  char **data;
  data = CreatePixmapDataFromFile(dpy, gc, fname);
  return(CreateMaskFromPixmapData(dpy, gc, data));
}

